package com.delfines.business;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import com.delfines.dao.ConnectionFactory;
import com.delfines.vo.ConsultaReferencia;
import com.delfines.vo.EstadoReferencia;
import com.delfines.vo.Referencia;
import com.delfines.vo.Usuario;

public class ReferenciaBo {
	
	public int existeReferenciaValidado(Integer idReferencia){
		int result=0; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT COUNT(1) cantidad FROM dbo.estado_referencia WHERE id_referencia=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idReferencia);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	result=rs.getInt("cantidad");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	
	public int existeNroTarjeta(String nroTarjeta){
		int result=0; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT COUNT(1) cantidad FROM dbo.referencia WHERE nro_trajeta=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nroTarjeta);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	result=rs.getInt("cantidad");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	
	public Integer actualizaNroRegistro(Integer idReferencia,Integer idUsuario){
		Integer result=0;
		
		PreparedStatement ps=null;
        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE referencia SET nro_registro=? WHERE id_usuario=? AND id_referencia=? ");
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idReferencia);
            ps.setInt(2, idUsuario);
            ps.setInt(3, idReferencia);
            
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	public Integer registroReferencia(Referencia referencia,Integer idUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=0;
        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO referencia(nro_registro,fecha_registro,apellidos_cliente,nombres_cliente,id_operador ");
            query.append(",nro_trajeta,correo_cliente,telefono_cliente,checkin_cliente,checkout_cliente ");
            query.append(",audit_usuarioid,audit_fechahora,audit_terminal,audit_estado,id_usuario,nickname) ");
            query.append("VALUES (NULL,GETDATE(),?,?,? ");
            query.append(",?,?,?,?,? ");
            query.append(",?,GETDATE(),NULL,'1',?,?) ");
            
            String ids[] = new String[] { "id_referencia" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setString(1, referencia.getApeCliente());
            ps.setString(2, referencia.getNomCliente());
            ps.setInt(3, referencia.getIdOperador());
            //--
            ps.setString(4, referencia.getNroTarjeta());
            ps.setString(5, referencia.getEmailCliente());
            ps.setString(6, referencia.getTelefonoCliente());
            //
            ps.setDate(7,Util.stringToDate(referencia.getCheckIn()));
            ps.setDate(8,Util.stringToDate(referencia.getCheckOut()));
            //
            ps.setInt(9, idUsuario);
            ps.setInt(10, idUsuario);
            ps.setString(11, referencia.getNickName());
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return pk;
	}
	
	public List<Referencia> buscarReferenciaByTarjeta(String nroTarjeta,Integer idUsuario){
		List<Referencia> lista = new ArrayList<Referencia>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append("WHERE r.id_usuario=? and r.nro_trajeta=? ");
			query.append(" ORDER BY r.id_referencia ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idUsuario);
            ps.setString(2, nroTarjeta);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	
            	referencia.setNickName(rs.getString("nickname"));
            	
            	referencia.setUsername(rs.getString("username"));
            	
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Referencia> buscarReferenciaByCliente(String apeCliente,String nomCliente,Usuario pUser){
		List<Referencia> lista = new ArrayList<Referencia>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append(", a.denominacion area ");//--
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append(" INNER JOIN dbo.area a ON (a.id_area=u.id_area) ");//--
			query.append("WHERE (r.apellidos_cliente LIKE '%"+apeCliente+"%' AND r.nombres_cliente LIKE '%"+nomCliente+"%') ");
			query.append(" AND r.id_referencia not in (select id_referencia from dbo.estado_referencia) ");
			query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append(" ORDER BY r.id_referencia ");
            
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	
            	referencia.setNickName(rs.getString("nickname"));
            	referencia.setUsername(rs.getString("username"));
            	
            	referencia.setArea(rs.getString("area"));//--
            	
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Referencia> buscarReferenciaByUsuario(Integer idUsuario,Usuario pUser){
		List<Referencia> lista = new ArrayList<Referencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append(", a.denominacion area ");//--
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append(" INNER JOIN dbo.area a ON (a.id_area=u.id_area) ");//--
			query.append("WHERE r.id_usuario=? ");
			query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append(" AND r.id_referencia not in (select id_referencia from dbo.estado_referencia) ");
			query.append(" ORDER BY r.id_referencia ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idUsuario);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	referencia.setNickName(rs.getString("nickname"));
            	referencia.setUsername(rs.getString("username"));
            	referencia.setArea(rs.getString("area"));//--
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Referencia> buscarReferenciaByNroRegistro(String nroRegistro,Usuario pUser){
		List<Referencia> lista = new ArrayList<Referencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append(", a.denominacion area ");//--
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append(" INNER JOIN dbo.area a ON (a.id_area=u.id_area) ");//--
			query.append("WHERE r.nro_registro=? ");
			query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append(" AND r.id_referencia not in (select id_referencia from dbo.estado_referencia) ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nroRegistro);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	referencia.setNickName(rs.getString("nickname"));
            	referencia.setUsername(rs.getString("username"));
            	referencia.setArea(rs.getString("area"));//--
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Referencia> buscarReferenciaByFecRegistro(String fechaRegistro,Usuario pUser){
		List<Referencia> lista = new ArrayList<Referencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append(", a.denominacion area ");//--
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append(" INNER JOIN dbo.area a ON (a.id_area=u.id_area) ");//--
			query.append("WHERE CONVERT(date,r.fecha_registro)=? ");
			query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append(" AND r.id_referencia not in (select id_referencia from dbo.estado_referencia) ");
			query.append(" ORDER BY r.id_referencia ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setDate(1, Util.stringToDate(fechaRegistro));
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	referencia.setNickName(rs.getString("nickname"));
            	referencia.setUsername(rs.getString("username"));
            	referencia.setArea(rs.getString("area"));//--
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}

	public List<Referencia> buscarReferenciaByNickName(String nickName,Usuario pUser){
		List<Referencia> lista = new ArrayList<Referencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
			query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
			query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,u.apellidos_nombres ape_nom_usuario,r.nickname,u.username ");
			query.append(", a.denominacion area ");//--
			query.append("FROM dbo.referencia r ");
			query.append(" INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append(" INNER JOIN dbo.area a ON (a.id_area=u.id_area) ");//--
			query.append("WHERE r.nickname=? ");
			query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append(" AND r.id_referencia not in (select id_referencia from dbo.estado_referencia) ");
			query.append(" ORDER BY r.id_referencia ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nickName);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Referencia referencia=new Referencia();
            	referencia.setIdReferencia(rs.getInt("id_referencia"));
            	referencia.setNroReferencia(rs.getString("nro_registro"));
            	referencia.setFechaRegistro(rs.getString("fecha_registro"));
            	
            	referencia.setApeCliente(rs.getString("apellidos_cliente"));
            	referencia.setNomCliente(rs.getString("nombres_cliente"));
            	
            	referencia.setIdOperador(rs.getInt("id_operador"));
            	referencia.setNroTarjeta(rs.getString("nro_trajeta"));
            	referencia.setEmailCliente(rs.getString("correo_cliente"));
            	referencia.setTelefonoCliente(rs.getString("telefono_cliente"));
            	
            	referencia.setCheckIn(rs.getString("checkin_cliente"));
            	referencia.setCheckOut(rs.getString("checkout_cliente"));
            	
            	referencia.setIdUsuario(rs.getInt("id_usuario"));
            	referencia.setApeNomUsuario(rs.getString("ape_nom_usuario"));
            	
            	referencia.setNickName(rs.getString("nickname"));
            	referencia.setUsername(rs.getString("username"));
            	referencia.setArea(rs.getString("area"));//--
            	lista.add(referencia);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	

	public int registroValidacion(EstadoReferencia referencia,Integer idUsuario){
		PreparedStatement ps=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO estado_referencia ");
            query.append("(entregado,validado,nick_name,fecha_comentario,titulo_comentario ");
            query.append(",detalle_comentario,punto_valoracion,id_operador,fecha_entregado,fecha_validado,id_referencia ");
            query.append(",audit_usuarioid,audit_fechahora,audit_terminal,audit_estado) ");
            query.append("VALUES (?,?,?,?,? ");
            query.append(",?,?,?,GETDATE(),GETDATE(),? ");
            query.append(",?,GETDATE(),NULL,'1') ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, referencia.getEntregado());
            ps.setString(2, referencia.getValidado());
            ps.setString(3, referencia.getNickName());
            ps.setDate(4, Util.stringToDate(referencia.getFechaComentario()));
            ps.setString(5, referencia.getTituloComentario());
            //--
            ps.setString(6, referencia.getDetalleComentario());
            ps.setString(7, referencia.getPuntoValoracion());
            ps.setInt(8, referencia.getIdOperador());
            ps.setInt(9, referencia.getIdReferencia());	
            ps.setInt(10, idUsuario);
            //--
            int i = ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return 1;
	}
	
	
	
	public List<ConsultaReferencia> buscarEstadoReferenciaByNroReferencia(Integer nroReferenciaIni,Integer nroReferenciaFin,Usuario pUser){
		List<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;
	
        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,e.id_estado_referencia,e.entregado,e.validado ");
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion ");
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area ");
            query.append(",a.denominacion ");
            query.append(",u.apellidos_nombres apellidos_nombres_usuario ");
            query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia ");
            query.append(", ISNULL(e.nick_name,'') nickname ");
            query.append(", u.username ");
            query.append("FROM referencia r ");
            query.append("INNER JOIN usuario u ON (u.id_usuario=r.id_usuario) ");
            query.append("INNER JOIN area a ON (a.id_area=u.id_area) ");
            query.append("LEFT JOIN estado_referencia e ON (r.id_referencia=e.id_referencia) ");
            query.append("WHERE r.nro_registro BETWEEN ? AND ? ");
            query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
            query.append(" ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, nroReferenciaIni.intValue());
            ps.setInt(2, nroReferenciaFin.intValue());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public ArrayList<ConsultaReferencia> buscarEstadoReferenciaByFechaRegistro(String fechaInicio,String fechaFin,Usuario pUser){
		ArrayList<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,ISNULL(e.id_estado_referencia,0) id_estado_referencia,e.entregado,e.validado ");
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion ");
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area ");
            query.append(",a.denominacion ");
            query.append(",u.apellidos_nombres apellidos_nombres_usuario ");
            query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia ");
            query.append(", ISNULL(e.nick_name,'') nickname ");
            query.append(", u.username ");
            query.append("FROM referencia r ");
            query.append("INNER JOIN usuario u ON (u.id_usuario=r.id_usuario) ");
            query.append("INNER JOIN area a ON (a.id_area=u.id_area) ");
            query.append("LEFT JOIN estado_referencia e ON (r.id_referencia=e.id_referencia) ");
            query.append("WHERE convert(date,r.fecha_registro) BETWEEN ? AND ? ");
            query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
            query.append(" ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setDate(1, Util.stringToDate(fechaInicio));
            ps.setDate(2, Util.stringToDate(fechaFin));
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<ConsultaReferencia> buscarEstadoReferenciaByUsuario(Integer idUsuario,Usuario pUser){
		List<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,e.id_estado_referencia,e.entregado,e.validado ");
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion ");
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area ");
            query.append(",a.denominacion ");
            query.append(",u.apellidos_nombres apellidos_nombres_usuario ");
            query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia ");
            query.append(", ISNULL(e.nick_name,'') nickname ");
            query.append(", u.username ");
            query.append("FROM referencia r ");
            query.append("INNER JOIN usuario u ON (u.id_usuario=r.id_usuario) ");
            query.append("INNER JOIN area a ON (a.id_area=u.id_area) ");
            query.append("LEFT JOIN estado_referencia e ON (r.id_referencia=e.id_referencia) ");
            query.append("WHERE r.id_usuario=? ");
            query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
            query.append(" ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idUsuario);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<ConsultaReferencia> buscarEstadoReferenciaByArea(Integer idArea,Usuario pUser){
		List<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,e.id_estado_referencia,e.entregado,e.validado ");//--
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion ");
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area ");
            query.append(",a.denominacion ");
            query.append(",u.apellidos_nombres apellidos_nombres_usuario ");
            query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia ");
            query.append(", ISNULL(e.nick_name,'') nickname ");
            query.append(", u.username ");
            query.append("FROM referencia r ");
            query.append("INNER JOIN usuario u ON (u.id_usuario=r.id_usuario) ");
            query.append("INNER JOIN area a ON (a.id_area=u.id_area) ");
            query.append("LEFT JOIN estado_referencia e ON (r.id_referencia=e.id_referencia) ");
            query.append("WHERE a.id_area=? ");
            query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
            query.append(" ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idArea);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<ConsultaReferencia> buscarEstadoReferenciaByNickName(String nickName,Usuario pUser){
		List<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente ");
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,e.id_estado_referencia,e.entregado,e.validado ");//--
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion ");
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area ");
            query.append(",a.denominacion ");
            query.append(",u.apellidos_nombres apellidos_nombres_usuario ");
            query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia ");
            query.append(", ISNULL(e.nick_name,'') nickname ");
            query.append(", u.username ");
            query.append("FROM referencia r ");
            query.append("INNER JOIN usuario u ON (u.id_usuario=r.id_usuario) ");
            query.append("INNER JOIN area a ON (a.id_area=u.id_area) ");
            query.append("LEFT JOIN estado_referencia e ON (r.id_referencia=e.id_referencia) ");
            query.append("WHERE e.nick_name=? ");
            query.append(sqlWherefromProfile(pUser.getPerfilId(), pUser.getAreaId(), pUser.getUsuarioId()));
            query.append(" ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nickName);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public String sqlWherefromProfile(Integer idPerfil,Integer idArea,Integer idUsuario){
		StringBuilder sqlWhere = new StringBuilder();
		
		if(idPerfil==1){
			sqlWhere.append(" AND u.id_area=").append(String.valueOf(idArea));
		}else if(idPerfil==2){
			sqlWhere.append(" AND u.id_area=").append(String.valueOf(idArea)).append(" AND u.id_usuario=").append(String.valueOf(idUsuario));
		}else if(idPerfil==9){
			sqlWhere.append(" AND 1=1 ");
		}
		
		return sqlWhere.toString();
	}
	
	public Integer retirarValidacion(Integer idReferencia,Integer idEstadoReferencia,String motivo,Integer usuarioId){
		CallableStatement cs = null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("{ call dbo.retira_validacion_referencia (?,?,?,?) }");
            
            cs = conn.prepareCall(query.toString());
            cs.setInt(1, idReferencia);
            cs.setInt(2, idEstadoReferencia);
            cs.setInt(3, usuarioId);
            cs.setString(4, motivo);
            //--
            cs.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (cs != null) {try {cs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return 1;
	}
	
	
	//Consulta de descargos
	public List<ConsultaReferencia> buscarEstadoDescargosByNroReferencia(Integer nroReferenciaIni,Integer nroReferenciaFin,Usuario pUser){
		List<ConsultaReferencia> lista = new ArrayList<ConsultaReferencia>();
		
		PreparedStatement ps=null;
		ResultSet rs=null;
	
        try {
            
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT r.id_referencia,r.nro_registro,dbo.getDateTimeAMPM(r.fecha_registro) fecha_registro,r.apellidos_cliente,r.nombres_cliente "); 
            query.append(",r.id_operador,r.nro_trajeta,r.correo_cliente,r.telefono_cliente,CONVERT(varchar,r.checkin_cliente,103) checkin_cliente  ");
            query.append(",CONVERT(varchar,r.checkout_cliente,103) checkout_cliente,r.id_usuario,e.id_estado_referencia,e.entregado,e.validado  ");
            query.append(",e.nick_name,CONVERT(varchar,e.fecha_comentario,103) fecha_comentario,e.titulo_comentario,e.detalle_comentario,e.punto_valoracion "); 
            query.append(",e.id_operador,dbo.getDateTimeAMPM(e.fecha_entregado) fecha_entregado,dbo.getDateTimeAMPM(e.fecha_validado) fecha_validado,e.id_referencia,a.id_area "); 
			query.append(",a.denominacion  ");
			query.append(",u.apellidos_nombres apellidos_nombres_usuario "); 
			query.append(", (CASE WHEN e.id_estado_referencia IS NULL THEN 'REGISTRADO' ELSE 'VALIDADO' END) estado_referencia "); 
			query.append(", ISNULL(e.nick_name,'') nickname  ");
			query.append(", u.username  ");
			query.append(", dbo.getDateTimeAMPM(o.fechahora_retiro) fechahora_retiro ");
			query.append(", uo.username username_retiro ");
			query.append(", o.motivo_retiro ");
			query.append("FROM dbo.referencia r "); 
			query.append("inner join dbo.estado_referencia_retiro o on (o.id_referencia=r.id_referencia) ");
			query.append("INNER JOIN dbo.usuario uo ON (uo.id_usuario=o.id_usuario_retiro) ");
			query.append("INNER JOIN dbo.usuario u ON (u.id_usuario=r.id_usuario) ");
			query.append("INNER JOIN dbo.area a ON (a.id_area=u.id_area)  ");
			query.append("LEFT JOIN dbo.estado_referencia e ON (r.id_referencia=e.id_referencia) "); 
			query.append("WHERE r.nro_registro BETWEEN ? AND ? ");
			query.append("ORDER BY r.id_referencia ASC ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, nroReferenciaIni.intValue());
            ps.setInt(2, nroReferenciaFin.intValue());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ConsultaReferencia consulta=new ConsultaReferencia(); 
            	
            	consulta.setIdReferencia(rs.getInt("id_referencia"));
            	consulta.setNroRegistro(rs.getString("nro_registro"));
            	consulta.setFechaRegistro(rs.getString("fecha_registro"));
            	consulta.setApellidosCliente(rs.getString("apellidos_cliente"));
            	consulta.setNombresCliente(rs.getString("nombres_cliente"));
            	consulta.setIdOperador(rs.getInt("id_operador"));
            	consulta.setNroTarjeta(rs.getString("nro_trajeta"));
            	consulta.setCorreoCliente(rs.getString("correo_cliente"));
            	consulta.setTelefonoCliente(rs.getString("telefono_cliente"));
            	consulta.setCheckinCliente(rs.getString("checkin_cliente"));
            	consulta.setCheckoutCliente(rs.getString("checkout_cliente"));
            	consulta.setIdUsuario(rs.getInt("id_usuario"));
            	consulta.setIdEstadoReferencia(rs.getInt("id_estado_referencia"));
            	consulta.setEntregado(rs.getString("entregado"));
            	consulta.setValidado(rs.getString("validado"));
            	consulta.setNick_name(rs.getString("nick_name"));
            	consulta.setFecha_comentario(rs.getString("fecha_comentario"));
            	consulta.setTitulo_comentario(rs.getString("titulo_comentario"));
            	consulta.setDetalle_comentario(rs.getString("detalle_comentario"));
            	consulta.setPuntoValoracion(rs.getInt("punto_valoracion"));
            	consulta.setFechaEntregado(rs.getString("fecha_entregado"));
            	consulta.setFechaValidado(rs.getString("fecha_validado"));
            	consulta.setIdArea(rs.getInt("id_area"));
            	consulta.setAreaDenominacion(rs.getString("denominacion"));
            	//--
            	consulta.setApellidosNombresUsuario(rs.getString("apellidos_nombres_usuario"));
            	consulta.setEstadoReferencia(rs.getString("estado_referencia"));
            	consulta.setNickName(rs.getString("nickname"));
            	consulta.setUsername(rs.getString("username"));
            	//--descargos de validaciones
            	consulta.setUsuarioDescargo(rs.getString("username_retiro"));
            	consulta.setFechaDescargo(rs.getString("fechahora_retiro"));
            	consulta.setMotivoDescargo(rs.getString("motivo_retiro"));
            	//--descargos de validaciones
            	
            	lista.add(consulta);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
}
