package com.delfines.business;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.delfines.Constante;
import com.delfines.dao.ConnectionFactory;
import com.delfines.vo.AdSetName;
import com.delfines.vo.Correo;
import com.delfines.vo.ErrorLead;
import com.delfines.vo.Lead;
import com.delfines.vo.LeadSeguimiento;
import com.delfines.vo.PerfilLead;
import com.delfines.vo.Usuario;
import com.delfines.vo.Value;
import com.delfines.vo.leads.ReporteCorreos;
import com.delfines.vo.leads.ReporteGeneral;

public class RegistroLeadsBo {
	
	public Map<String, Integer> loadLeads(String filePath){
		CallableStatement cs = null;
		Integer errors=Constante.ID_INICIAL;
		Integer leads=Constante.ID_INICIAL;
		
		Map<String, Integer> result=new HashMap<String,Integer>();
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("{ call leads.load_leads_from_csv_2(?,?) }");
            
            cs = conn.prepareCall(query.toString());
            cs.registerOutParameter(1, java.sql.Types.INTEGER);  
            cs.registerOutParameter(2, java.sql.Types.INTEGER);
            //--
            cs.execute();
            errors=cs.getInt(1);
            leads=cs.getInt(2);
            
            System.out.println("load errors: " + errors);  
            System.out.println("load leads: " + leads);
            
            result.put("errors", errors);
            result.put("leads", leads);
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (cs != null) {try {cs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	public List<ErrorLead> listLeadsErrors(){
		List<ErrorLead> lista = new ArrayList<ErrorLead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT e.ERROR_NUMBER, ");
            query.append("e.ERROR_MESSAGE, ");
            query.append("e.ERROR_DATE, ");
            query.append("e.id_adset_name_1, ");
            query.append("e.dia_adset_2, ");
            query.append("e.mes_adset_3, ");
            query.append("e.anyo_adset_4, ");
            query.append("e.pax_adulto_5, ");
            query.append("e.pax_ninyo_6, ");
            query.append("e.nombre_cliente_8, ");
            query.append("e.email_cliente_9, ");
            query.append("e.telefono_cliente_10 ");
            query.append("FROM leads.ERROR_LEAD e ");
		
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ErrorLead obj=new ErrorLead();
            	obj.setERROR_NUMBER(rs.getString("ERROR_NUMBER"));
            	obj.setERROR_MESSAGE(rs.getString("ERROR_MESSAGE"));
            	obj.setERROR_DATE(rs.getString("ERROR_DATE"));
            	obj.setId_adset_name_1(rs.getString("id_adset_name_1"));
            	obj.setDia_adset_2(rs.getString("dia_adset_2"));
            	obj.setMes_adset_3(rs.getString("mes_adset_3"));
            	obj.setAnyo_adset_4(rs.getString("anyo_adset_4"));
            	obj.setPax_adulto_5(rs.getString("pax_adulto_5"));
            	obj.setPax_ninyo_6(rs.getString("pax_ninyo_6"));
            	obj.setNombre_cliente_8(rs.getString("nombre_cliente_8"));
            	obj.setEmail_cliente_9(rs.getString("email_cliente_9"));
            	obj.setTelefono_cliente_10(rs.getString("telefono_cliente_10"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Lead> listLeadsByIdAdSetName(Integer idAdSetName,Usuario pUser){
		List<Lead> lista = new ArrayList<Lead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select l.id_lead,a.denominacion adsetname,convert(varchar(20),l.fecha_adset,103) fecha_adset,l.pax_adulto,l.pax_ninyo,l.nombre_cliente, "); 
            query.append("l.email_cliente,l.telefono_cliente,l.precio_pax_adulto,l.precio_pax_ninyo,r.denominacion area,s.denominacion sub_area, "); 
            query.append("l.id_area,l.id_sub_area ");
            query.append(",e.username ejecutivo ");
            query.append(",e.id_usuario id_ejecutiva ");
            query.append(",(SELECT count(1) FROM leads.lead_seguimiento s where s.id_lead=l.id_lead) seguimiento ");
            query.append(",dbo.getEstadoLead(l.id_lead) estado_lead ");
            query.append("from leads.lead l  ");
            query.append("inner join leads.adset_name a on (l.id_adset_name=a.id_adset_name) "); 
            query.append("inner join dbo.area r on (r.id_area=l.id_area)  ");
            query.append("inner join leads.sub_area s on (s.id_sub_area=l.id_sub_area) ");
            query.append("left  join dbo.usuario e on (e.id_usuario=l.id_ejecutiva) ");
            query.append("where l.id_adset_name=?  ");
            query.append(sqlWherefromProfile(pUser.getIdPerfilLead(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append("ORDER BY l.id_lead ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idAdSetName);
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Lead obj=new Lead();
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setAdSetName(rs.getString("adsetname"));
            	obj.setFechaAdSet(rs.getString("fecha_adset"));
            	obj.setPaxAdulto(rs.getInt("pax_adulto"));
            	obj.setPaxNinyo(rs.getInt("pax_ninyo"));
            	obj.setNombreCliente(rs.getString("nombre_cliente"));
            	obj.setEmailCliente(rs.getString("email_cliente"));
            	obj.setTelefonoCliente(rs.getString("telefono_cliente"));
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto"));
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setArea(rs.getString("area"));
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setSubArea(rs.getString("sub_area"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	obj.setIdEjecutivo(rs.getInt("id_ejecutiva"));
            	obj.setSeguimiento(rs.getInt("seguimiento"));
            		obj.setEstadoLead(rs.getInt("estado_lead"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Lead> listLeads(Integer idAdSetName,String fechaCargaIni,String fechaCargaFin,Integer idArea,Integer idSubArea,Integer idEstado,
			String fechaLeadIni,String fechaLeadFin,String fechaProxIni,String fechaProxFin,Integer idSeguimiento,
			Integer idMotivo,Integer idReserva,Integer seqSeguimiento,Integer idCotizado,
			Usuario pUser){
		List<Lead> lista = new ArrayList<Lead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select l.id_lead,a.denominacion adsetname,convert(varchar(20),l.fecha_adset,103) fecha_adset,l.pax_adulto,l.pax_ninyo,l.nombre_cliente, "); 
            query.append("l.email_cliente,l.telefono_cliente,l.precio_pax_adulto,l.precio_pax_ninyo,r.denominacion area,s.denominacion sub_area, "); 
            query.append("l.id_area,l.id_sub_area ");
            query.append(",e.username ejecutivo ");
            query.append(",e.id_usuario id_ejecutiva ");
            query.append(",(SELECT count(1) FROM leads.lead_seguimiento s where s.id_lead=l.id_lead) seguimiento ");
            query.append(",dbo.getEstadoLead(l.id_lead) estado_lead ");
            query.append(",convert(varchar(20),l.fecha_carga,103) fecha_carga ");
            query.append(",dbo.getTotalPax(a.es_individual,l.pax_adulto,a.precio_pax_adulto)+dbo.getTotalPax(a.es_individual,l.pax_ninyo,a.precio_pax_ninyo) total_reserva ");
            query.append(",convert(varchar(20),ls.fecha_proxima,103) fecha_proxima ");
            query.append(",dbo.getHora(ls.hora) hora ");
            query.append(",dbo.getMotivo(ls.id_motivo) motivo ");
            query.append(",dbo.getReserva(ls.id_reserva) reserva ");
            query.append(",dbo.getCotiza(ls.id_cotizado) cotizado ");
            query.append(",isnull(ls.seq_seguimiento,0) seq_seguimiento ");
            query.append(",convert(varchar(20),ls.fecha_contacto,103) fecha_contacto   ");
            query.append(",dbo.getCanal(ls.id_canal) canal ");
            query.append(",ISNULL(ls.pax_final,0) pax_final ");
            query.append(",ISNULL(ls.revenue,0) revenue ");
            query.append("from leads.lead l ");
            query.append("inner join leads.adset_name a on (l.id_adset_name=a.id_adset_name) "); 
            query.append("inner join leads.sub_area s on (s.id_sub_area=l.id_sub_area) ");
            query.append("inner join dbo.area r on (r.id_area=s.id_area)   ");
            query.append("left  join dbo.usuario e on (e.id_usuario=l.id_ejecutiva) ");
            query.append("left join leads.lead_seguimiento ls on (ls.id_lead=l.id_lead and ls.estado='1') ");
            query.append("where 1=1  ");
            
            if(idAdSetName!=null && idAdSetName>0){
            	query.append("and l.id_adset_name=?  ");		   
     	    }
            
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
     			   fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
     	    	query.append("and l.fecha_carga between ? and ? ");
     	    }
            
            if(fechaLeadIni!=null&&fechaLeadIni.trim().length()>0&&
      			   fechaLeadFin!=null&&fechaLeadFin.trim().length()>0){
      	    	query.append("and l.fecha_adset between ? and ? ");
      	    }
            
            if(fechaProxIni!=null&&fechaProxIni.trim().length()>0&&
       			   fechaProxFin!=null&&fechaProxFin.trim().length()>0){
       	    	query.append("and ls.fecha_proxima between ? and ? ");
       	    }
            
            if(idArea!=null && idArea>0){
     	    	query.append("and l.id_area=? ");
     	    	if(idSubArea!=null && idSubArea>0){
     	    		query.append("and l.id_sub_area=? ");
     	    	}
     	    }
            if(idEstado!=null && idEstado<9){
            	if(idEstado==0){//NO ASIGNADO
            		query.append("and ISNULL(l.id_ejecutiva,0)=? ");	
            	}else if(idEstado==1){//SI ASIGNADO
            		query.append("and ISNULL(l.id_ejecutiva,0)>=? ");
            	}
            }
            
            if(idSeguimiento!=null && idSeguimiento<9){
            	if(idSeguimiento==0){//NO ASIGNADO
            		query.append("and ISNULL(ls.id_lead_seguimiento,0)=? ");	
            	}else if(idSeguimiento==1){//SI ASIGNADO
            		query.append("and ISNULL(ls.id_lead_seguimiento,0)>=? ");
            	}
            }
            
            //nuevos filtros 28/082018
            if(idMotivo!=null && idMotivo>0){
            	query.append("and ls.id_motivo=? ");		   
     	    }
            if(idReserva!=null && idReserva>0){
            	query.append("and ls.id_reserva=? ");		   
     	    }
            if(seqSeguimiento!=null && seqSeguimiento<99){
            	query.append("and ls.seq_seguimiento=? ");		   
     	    }
            if(idCotizado!=null && idCotizado>0){
            	query.append("and ISNULL(ls.id_cotizado,0)=? ");		   
     	    }
            
            query.append(sqlWherefromProfile(pUser.getIdPerfilLead(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append("ORDER BY l.id_lead DESC ");
			
			Integer index=1;
			
            ps = conn.prepareStatement(query.toString());
            if(idAdSetName!=null && idAdSetName>0){
            	ps.setInt(index++, idAdSetName);		   
     	    }
            
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
     			   fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
     	    	ps.setDate(index++, Util.stringToDate(fechaCargaIni));
     	    	ps.setDate(index++, Util.stringToDate(fechaCargaFin));
     	    }
            
            if(fechaLeadIni!=null&&fechaLeadIni.trim().length()>0&&
      			   fechaLeadFin!=null&&fechaLeadFin.trim().length()>0){
      	    	ps.setDate(index++, Util.stringToDate(fechaLeadIni));
      	    	ps.setDate(index++, Util.stringToDate(fechaLeadFin));
      	    }
            
            if(fechaProxIni!=null&&fechaProxIni.trim().length()>0&&
       			   fechaProxFin!=null&&fechaProxFin.trim().length()>0){
       	    	ps.setDate(index++, Util.stringToDate(fechaProxIni));
       	    	ps.setDate(index++, Util.stringToDate(fechaProxFin));
       	    }
            
            if(idArea!=null && idArea>0){
     	    	ps.setInt(index++, idArea);
     	    	if(idSubArea!=null && idSubArea>0){
     	    		ps.setInt(index++, idSubArea);
     	    	}
     	    }
            
            if(idEstado!=null && idEstado<9){
            	if(idEstado==0){//NO ASIGNADO
            		ps.setInt(index++, idEstado);	
            	}else if(idEstado==1){//SI ASIGNADO
            		ps.setInt(index++, idEstado);
            	}
     	    }
            
            if(idSeguimiento!=null && idSeguimiento<9){
            	if(idSeguimiento==0){//NO ASIGNADO
            		ps.setInt(index++, idSeguimiento);	
            	}else if(idSeguimiento==1){//SI ASIGNADO
            		ps.setInt(index++, idSeguimiento);
            	}
     	    }
            
            //nuevos filtros 28/082018
            if(idMotivo!=null && idMotivo>0){
            	ps.setInt(index++, idMotivo);		   
     	    }
            if(idReserva!=null && idReserva>0){
            	ps.setInt(index++, idReserva);		   
     	    }
            if(seqSeguimiento!=null && seqSeguimiento<99){
            	ps.setInt(index++, seqSeguimiento);
     	    }
            if(idCotizado!=null && idCotizado>0){
            	ps.setInt(index++, idCotizado);		   
     	    }
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Lead obj=new Lead();
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setAdSetName(rs.getString("adsetname"));
            	obj.setFechaAdSet(rs.getString("fecha_adset"));
            	obj.setPaxAdulto(rs.getInt("pax_adulto"));
            	obj.setPaxNinyo(rs.getInt("pax_ninyo"));
            	obj.setNombreCliente(rs.getString("nombre_cliente"));
            	obj.setEmailCliente(rs.getString("email_cliente"));
            	obj.setTelefonoCliente(rs.getString("telefono_cliente"));
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto"));
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setArea(rs.getString("area"));
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setSubArea(rs.getString("sub_area"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	obj.setIdEjecutivo(rs.getInt("id_ejecutiva"));
            	obj.setSeguimiento(rs.getInt("seguimiento"));
            	obj.setEstadoLead(rs.getInt("estado_lead"));
            	
            	obj.setFechaCarga(rs.getString("fecha_carga"));
            	
            	obj.setTotalReserva(rs.getDouble("total_reserva"));
            	
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	obj.setHora(rs.getString("hora"));
            	//nuevos filtros 28/082018
            	obj.setMotivo(rs.getString("motivo"));
            	obj.setReserva(rs.getString("reserva"));
            	obj.setSeqSeguimiento(rs.getInt("seq_seguimiento"));
            	
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	//campos pax y revenue 31/08
            	obj.setPaxFinal(rs.getInt("pax_final"));
            	obj.setRevenue(rs.getDouble("revenue"));
            	
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Lead> listLeadsByFechaAdSet(String fechaIni,String fechaFin,Usuario pUser){
		List<Lead> lista = new ArrayList<Lead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select l.id_lead,a.denominacion adsetname,convert(varchar(20),l.fecha_adset,103) fecha_adset,l.pax_adulto,l.pax_ninyo,l.nombre_cliente, "); 
            query.append("l.email_cliente,l.telefono_cliente,l.precio_pax_adulto,l.precio_pax_ninyo,r.denominacion area,s.denominacion sub_area, "); 
            query.append("l.id_area,l.id_sub_area "); 
            query.append(",e.username ejecutivo ");
            query.append(",e.id_usuario id_ejecutiva ");
            query.append(",(SELECT count(1) FROM leads.lead_seguimiento s where s.id_lead=l.id_lead) seguimiento ");
            query.append(",dbo.getEstadoLead(l.id_lead) estado_lead ");
            query.append("from leads.lead l  ");
            query.append("inner join leads.adset_name a on (l.id_adset_name=a.id_adset_name) "); 
            query.append("inner join dbo.area r on (r.id_area=l.id_area)  ");
            query.append("inner join leads.sub_area s on (s.id_sub_area=l.id_sub_area) "); 
            query.append("left join dbo.usuario e on (e.id_usuario=l.id_ejecutiva) ");
            query.append("where l.fecha_carga between ? and ? ");
            query.append(sqlWherefromProfile(pUser.getIdPerfilLead(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append("ORDER BY l.id_lead ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setDate(1, Util.stringToDate(fechaIni));
            ps.setDate(2, Util.stringToDate(fechaFin));
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Lead obj=new Lead();
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setAdSetName(rs.getString("adsetname"));
            	obj.setFechaAdSet(rs.getString("fecha_adset"));
            	obj.setPaxAdulto(rs.getInt("pax_adulto"));
            	obj.setPaxNinyo(rs.getInt("pax_ninyo"));
            	obj.setNombreCliente(rs.getString("nombre_cliente"));
            	obj.setEmailCliente(rs.getString("email_cliente"));
            	obj.setTelefonoCliente(rs.getString("telefono_cliente"));
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto"));
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setArea(rs.getString("area"));
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setSubArea(rs.getString("sub_area"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	obj.setIdEjecutivo(rs.getInt("id_ejecutiva"));
            	obj.setSeguimiento(rs.getInt("seguimiento"));
            		obj.setEstadoLead(rs.getInt("estado_lead"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Lead> listLeadsByIdArea(Integer idArea,Integer idSubArea,Usuario pUser){
		List<Lead> lista = new ArrayList<Lead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select l.id_lead,a.denominacion adsetname,convert(varchar(20),l.fecha_adset,103) fecha_adset,l.pax_adulto,l.pax_ninyo,l.nombre_cliente, "); 
            query.append("l.email_cliente,l.telefono_cliente,l.precio_pax_adulto,l.precio_pax_ninyo,r.denominacion area,s.denominacion sub_area, "); 
            query.append("l.id_area,l.id_sub_area "); 
            query.append(",e.username ejecutivo ");
            query.append(",e.id_usuario id_ejecutiva ");
            query.append(",(SELECT count(1) FROM leads.lead_seguimiento s where s.id_lead=l.id_lead) seguimiento ");
            query.append(",dbo.getEstadoLead(l.id_lead) estado_lead ");
            query.append("from leads.lead l  ");
            query.append("inner join leads.adset_name a on (l.id_adset_name=a.id_adset_name) "); 
            query.append("inner join dbo.area r on (r.id_area=l.id_area)  ");
            query.append("inner join leads.sub_area s on (s.id_sub_area=l.id_sub_area) "); 
            query.append("left  join dbo.usuario e on (e.id_usuario=l.id_ejecutiva) ");
            query.append("where l.id_area=? and l.id_sub_area=? ");
            query.append(sqlWherefromProfile(pUser.getIdPerfilLead(), pUser.getAreaId(), pUser.getUsuarioId()));
			query.append("ORDER BY l.id_lead ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idArea);
            ps.setInt(2, idSubArea);
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Lead obj=new Lead();
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setAdSetName(rs.getString("adsetname"));
            	obj.setFechaAdSet(rs.getString("fecha_adset"));
            	obj.setPaxAdulto(rs.getInt("pax_adulto"));
            	obj.setPaxNinyo(rs.getInt("pax_ninyo"));
            	obj.setNombreCliente(rs.getString("nombre_cliente"));
            	obj.setEmailCliente(rs.getString("email_cliente"));
            	obj.setTelefonoCliente(rs.getString("telefono_cliente"));
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto"));
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setArea(rs.getString("area"));
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setSubArea(rs.getString("sub_area"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	obj.setIdEjecutivo(rs.getInt("id_ejecutiva"));
            	obj.setSeguimiento(rs.getInt("seguimiento"));
            	obj.setEstadoLead(rs.getInt("estado_lead"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return lista;
	}
	
	public List<LeadSeguimiento> listLeadsSeguimientoByIdLead(Integer idLead){
		List<LeadSeguimiento> lista = new ArrayList<LeadSeguimiento>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_lead_seguimiento ");
            query.append(",(case when id_cotizado=1 then 'SI' when id_cotizado=2 then 'NO' when id_cotizado=3 then 'CERRADO' END) cotizado "); 
            query.append(",convert(varchar(20),fecha_contacto,103) fecha_contacto  ");
            query.append(",(case when id_canal=1 then 'CORREO' WHEN id_canal=2 THEN 'LLAMADA' WHEN id_canal=3 THEN 'PRESENCIAL'  WHEN id_canal=4 THEN 'WHATSAPP'  ELSE 'NINGUNO' END) canal "); 
            query.append(",(case when id_reserva=1 then 'SI' WHEN id_reserva=3 then 'Seg' ELSE 'NO' END) reserva  ");
            query.append(",(CASE   ");
            query.append("WHEN id_motivo=1 then 'NO CONTESTA' "); 
            query.append("WHEN id_motivo=2 then 'NO ESTA INTERESADO' "); 
            query.append("WHEN id_motivo=3 then 'POR CONFIRMAR'  ");
            query.append("WHEN id_motivo=4 then 'NUMERO INVALIDO'  ");
            query.append("WHEN id_motivo=5 then 'VOLVER A CONTACTAR'  ");
            query.append("WHEN id_motivo=6 then 'TOMO OTRA OPCION'  ");
            query.append("WHEN id_motivo=7 then 'SI RESERVA'  ");
            query.append("ELSE 'NINGUNO'  ");
            query.append("END) motivo  ");
            query.append(",pax_final  ");
            query.append(",revenue  ");
            query.append(",( SELECT denominacion  FROM leads.adset_name where id_adset_name=s.id_adset_name_final ) servicio_final ");
            query.append(",e.id_usuario  ");
            query.append(",s.id_lead  ");
            query.append(",e.username ejecutivo "); 
            query.append(",s.observacion ");
            query.append(",convert(varchar(20),s.fecha_proxima,103) fecha_proxima ");
            query.append(",s.seq_seguimiento ");
            query.append(",s.estado ");
            query.append(",dbo.getHora(s.hora) hora ");
            query.append("FROM leads.lead_seguimiento s  ");
            query.append("inner join dbo.usuario e on (e.id_usuario=s.id_usuario) "); 
            query.append("where s.id_lead=? ORDER BY id_lead_seguimiento ASC    ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idLead);
            
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	LeadSeguimiento obj=new LeadSeguimiento();
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setIdLeadSeguimiento(rs.getInt("id_lead_seguimiento"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	obj.setReserva(rs.getString("reserva"));
            	obj.setMotivo(rs.getString("motivo"));
            	obj.setPaxFinal(rs.getString("pax_final"));
            	obj.setRevenue(rs.getString("revenue"));
            	obj.setServicioFinal(rs.getString("servicio_final"));
            	obj.setIdUsuario(rs.getInt("id_usuario"));
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	//--
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	obj.setObservacion(rs.getString("observacion"));
            	obj.setSecuencia(rs.getInt("seq_seguimiento"));
            	obj.setEstado(rs.getString("estado"));
            	
            	obj.setHora(rs.getString("hora"));
            	
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Value> listAdSetName(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select id_adset_name id, denominacion from leads.adset_name where estado='1'  order by denominacion asc ");
		
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	public List<Value> listArea(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select id_area id, denominacion from dbo.area where id_area IN (1,6,9,17) order by denominacion asc ");//Ventas//A & B//Eventos y Banquetes
		
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion").toUpperCase());
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Value> listSubArea(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select id_sub_area id, denominacion from leads.sub_area where estado='1' and  id_area IN (1,6,9) order by denominacion asc");
		
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}

	public List<Value> listEjecutivo(Integer idPerfil,Integer idArea){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT e.id_usuario id,e.apellidos_nombres denominacion FROM dbo.usuario e "); 
            query.append("inner join leads.usuario_perfil p on (e.id_usuario=p.id_usuario) ");
            query.append("where p.id_modulo=2 and p.id_perfil IN (?) and e.audit_estado=1 ");
            if(idArea!=0){
            	query.append(" AND e.id_area=? ");
            }
            query.append("order by e.apellidos_nombres asc ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idPerfil);
            if(idArea!=0){
            	ps.setInt(2, idArea);	
            }
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Value> listEjecutivoCorreo(Integer idPerfil,Integer idArea,Integer idUsuario){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT e.id_usuario id,e.apellidos_nombres denominacion FROM dbo.usuario e "); 
            query.append("inner join leads.usuario_perfil p on (e.id_usuario=p.id_usuario) ");
            query.append("where p.id_modulo=2  ");
            query.append(sqlWherefromProfileCorreo(idPerfil, idArea,idUsuario));
            query.append("order by e.apellidos_nombres asc ");
            
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public String sqlWherefromProfileCorreo(Integer idPerfil,Integer idArea,Integer idUsuario){
		StringBuilder sqlWhere = new StringBuilder();
		
		if(idPerfil==1)//<option value="1">ADMINISTRA</option>
		{
			sqlWhere.append(" AND p.id_perfil IN (6,2) ");//<option value="6">ASIGNA-CORREO</option> // <option value="2">ASIGNA</option> 
		}else if((idPerfil==6)||(idPerfil==2)){
			sqlWhere.append(" AND p.id_perfil IN (5,3) AND p.id_categoria="+idArea+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
		}else if((idPerfil==5)||(idPerfil==3)){
			sqlWhere.append(" AND p.id_perfil IN (5,3) AND e.id_usuario="+idUsuario+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
		}else{
			sqlWhere.append(" AND 1!=1 ");
		}
		
		return sqlWhere.toString();
	}
	
	
	public String sqlWherefromProfile(Integer idLeadPerfil,Integer idArea,Integer idUsuario){
		StringBuilder sqlWhere = new StringBuilder();
		
		if(idLeadPerfil==1){
			sqlWhere.append(" AND 1=1 ");
		}else if(idLeadPerfil==2){
			sqlWhere.append(" AND 1=1 ");
		}else if(idLeadPerfil==4){
			sqlWhere.append(" AND 1=1 ");
		}else {//if(idLeadPerfil==3)
			sqlWhere.append(" AND l.id_ejecutiva=").append(String.valueOf(idUsuario));
		}
		
		return sqlWhere.toString();
	}
	
	
	public Integer reAsignarEjecutivoLead(Integer idEjecutivo,Integer idLead){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.lead   SET id_ejecutiva = ? WHERE id_lead=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idEjecutivo);
            ps.setInt(2, idLead);
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	
	public Integer actualizarFechaServicio(Integer idLead,String fechaAdset){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.lead   SET fecha_adset = ? WHERE id_lead=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setDate(1, Util.stringToDate(fechaAdset));
            ps.setInt(2, idLead);
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	
	/*
	if(idPerfil==1)//<option value="1">ADMINISTRA</option>
	{
		sqlWhere.append(" AND p.id_perfil IN (6,2) ");//<option value="6">ASIGNA-CORREO</option> // <option value="2">ASIGNA</option> 
	}else if((idPerfil==6)||(idPerfil==2)){
		sqlWhere.append(" AND p.id_perfil IN (5,3) AND p.id_categoria="+idArea+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
	}else if((idPerfil==5)||(idPerfil==3)){
		sqlWhere.append(" AND p.id_perfil IN (5,3) AND e.id_usuario="+idUsuario+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
	}else{
		sqlWhere.append(" AND 1!=1 ");
	}
	*/
	
	public Integer reAsignarEjecutivoEmail(Integer idEjecutivo,Integer idLeadPerfil,Integer idEmail){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            if(idLeadPerfil==1){//Administrador
            	query.append("UPDATE dbo.email SET id_ejecutiva = ?,id_area=(select id_area from dbo.usuario where id_usuario=? and audit_estado='1'),fecha_asigna_area=getdate() WHERE id_email =? ");
            }else{
            	query.append("UPDATE dbo.email SET id_ejecutiva = ?,id_area=(select id_area from dbo.usuario where id_usuario=? and audit_estado='1'),audit_fechahora=getdate() WHERE id_email =? ");
            }
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idEjecutivo);
            ps.setInt(2, idEjecutivo);
            ps.setInt(3, idEmail);
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	
	public Integer asignarEjecutivo(Integer idEjecutivo,List<Lead> listaLeads,Integer desde,Integer hasta){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
        	String Ids=getIdLeads(listaLeads,desde,hasta);
        	
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.lead SET id_ejecutiva = ? WHERE id_lead IN ("+Ids+") ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idEjecutivo);
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return result;
	}
	
	public String getIdLeads(List<Lead> listaLeads,Integer desde,Integer hasta){
		StringBuilder ids = new StringBuilder();
		
		List<Lead> subListaLeads=listaLeads.subList(desde==0?0:(desde-1),hasta);
		
		for (Iterator<Lead> it = subListaLeads.iterator(); it.hasNext(); ) {
		    if (it.hasNext()) {
		    	ids.append(it.next().getIdLead());
		    	ids.append(",");
		    }
		}
		ids.append("0");
		
		return ids.toString();
	}
	
	public Integer guardarSeguimiento(Integer idEjecutivo,Integer idLead,Integer idCotizado,Integer idCanal,
			Integer idReserva,Integer idMotivo,Integer paxFinal,Double revenue,Integer idAdSetNameFinal,String observacion,String fechaProxima,Integer hora,
			Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO leads.lead_seguimiento(id_cotizado,fecha_contacto,id_canal,id_reserva,id_motivo,pax_final,revenue,id_adset_name_final,id_usuario,id_lead,observacion,fecha_proxima,seq_seguimiento,estado,hora) ");
            query.append("VALUES(?,getdate(),?,?,?,?,?,?,?,?,?,?,?,?,?) ");
            String ids[] = new String[] { "id_lead_seguimiento" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setInt(1, idCotizado);
            ps.setInt(2, idCanal);
            ps.setInt(3, idReserva);
            ps.setInt(4, idMotivo);
            ps.setInt(5, paxFinal==null?0:paxFinal);
            ps.setDouble(6, revenue==null?0.0:revenue);
            ps.setInt(7, idAdSetNameFinal);
            ps.setInt(8, idEjecutivo);
            ps.setInt(9, idLead);
            ps.setString(10, observacion==null?"":observacion);
            if(fechaProxima.trim().length()>0){
            	ps.setDate(11, Util.stringToDate(fechaProxima));	
            }else{
            	ps.setNull(11, java.sql.Types.DATE);
            }
            
            ps.setInt(12, Constante.ESTADO_ACTIVO);//1
            ps.setString(13, String.valueOf(Constante.ESTADO_ACTIVO));//'1'
            if(hora!=null){
            	ps.setInt(14, hora);	
            }else{
            	ps.setNull(14, java.sql.Types.INTEGER);
            }
            
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
		return pk;
	}
	
	
	public Integer actualizarSeguimiento(Integer idLeadSeguimiento,Integer idLead,Integer idCotizado,Integer idCanal,
			Integer idReserva,Integer idMotivo,Integer paxFinal,Double revenue,Integer idAdSetNameFinal,String observacion,String fechaProxima,Integer hora,
			Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.lead_seguimiento ");
            query.append("SET id_cotizado = ? ");
            query.append(",id_canal = ? ");
            query.append(",id_reserva = ? ");
            query.append(",id_motivo = ? ");
            query.append(",pax_final = ? ");
            query.append(",revenue = ? ");
            query.append(",observacion = ? ");
            query.append(",id_adset_name_final = ? ");
            query.append(",fecha_proxima = ? ");
            query.append(",hora = ? ");
            query.append("WHERE id_lead_seguimiento=? and id_lead=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idCotizado);
            ps.setInt(2, idCanal);
            ps.setInt(3, idReserva);
            ps.setInt(4, idMotivo);
            ps.setInt(5, paxFinal==null?0:paxFinal);
            ps.setDouble(6, revenue==null?0.0:revenue);
            ps.setString(7, observacion==null?"":observacion);
            ps.setInt(8, idAdSetNameFinal);
            if(fechaProxima.trim().length()>0){
            	ps.setDate(9, Util.stringToDate(fechaProxima));	
            }else{
            	ps.setNull(9, java.sql.Types.DATE);
            }
            if(hora!=null){
            	ps.setInt(10, hora);	
            }else{
            	ps.setNull(10, java.sql.Types.INTEGER);
            }
            ps.setInt(11, idLeadSeguimiento);
            ps.setInt(12, idLead);
            //--
            if(ps.executeUpdate()>0){
            	pk =idLeadSeguimiento;
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
		return pk;
	}
	
	
	public Integer actualiaEstadoSeguimiento(Integer idLeadSeguimiento){
		CallableStatement cs = null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("{ call leads.estado_lead_seguimiento(?) }");
            
            cs = conn.prepareCall(query.toString());
            cs.setInt(1, idLeadSeguimiento);
            
            cs.execute();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (cs != null) {try {cs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return 1;
	}
	

	public String getDate(){
		String fecha=""; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT convert(varchar(20),getdate(),103) hoy ");
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	fecha=rs.getString("hoy");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return fecha;
	}
	
	public String getDateTime(){
		String fecha=""; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT dbo.getDateTimeAMPM(getdate()) hoy ");
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	fecha=rs.getString("hoy");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return fecha;
	}
	
	/**
	 * mantenimiento de leads
	 */
	
	public Integer actualizaAds(Integer id,String denominacion,String descripcion,
			Integer idArea,Integer idSubArea,Double precioPaqAdulto,Double precioPaqNinyo,Integer esIndividual,
			Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.adset_name ");
            query.append("SET denominacion = ? ");
            query.append(",descripcion = ? ");
            query.append(",id_sub_area = ? ");
            query.append(",id_area = ? ");
            query.append(",precio_pax_adulto = ? ");
            query.append(",precio_pax_ninyo = ? ");
            query.append(",es_individual=? ");
            query.append("WHERE id_adset_name=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, denominacion);
            ps.setString(2, descripcion);
            ps.setInt(3, idSubArea);
            ps.setInt(4, idArea);
            ps.setDouble(5, precioPaqAdulto);
            ps.setDouble(6, precioPaqNinyo);
            ps.setInt(7, esIndividual);
            ps.setInt(8, id);
            //--
            res=ps.executeUpdate();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	
	public Integer registraAds(String denominacion,String descripcion,
			Integer idArea,Integer idSubArea,Double precioPaqAdulto,Double precioPaqNinyo,Integer esIndividual,
			Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO leads.adset_name(denominacion,descripcion,id_area,id_sub_area,precio_pax_adulto,precio_pax_ninyo,estado,es_individual) VALUES (?,?,?,?,?,?,?,?) ");
            String ids[] = new String[] { "id_adset_name" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setString(1, denominacion);
            ps.setString(2, descripcion);
            ps.setInt(3, idArea);
            ps.setInt(4, idSubArea);
            ps.setDouble(5, (precioPaqAdulto!=null?precioPaqAdulto:0));
            ps.setDouble(6, (precioPaqNinyo!=null?precioPaqNinyo:0));
            ps.setString(7, String.valueOf(Constante.ESTADO_ACTIVO));
            ps.setInt(8, esIndividual);
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return pk;
	}
	
	public Integer eliminarAds(Integer id,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.adset_name SET estado = ?  WHERE id_adset_name=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, String.valueOf(Constante.ESTADO_ELIMINADO));
            ps.setInt(2, id);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	public List<AdSetName> listaAds(){
		List<AdSetName> lista = new ArrayList<AdSetName>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT d.id_adset_name ");
            query.append(",d.denominacion ");
            query.append(",d.descripcion ");
            query.append(",s.id_sub_area ");
            query.append(",s.denominacion sub_area ");
            query.append(",a.id_area ");
            query.append(",a.denominacion area ");
            query.append(",d.precio_pax_adulto ");
            query.append(",d.precio_pax_ninyo ");
            query.append(",d.estado ");
            query.append(",d.es_individual ");
            query.append("FROM leads.adset_name d ");
            query.append("INNER JOIN leads.sub_area s ON (s.id_sub_area=d.id_sub_area) ");
            query.append("INNER JOIN dbo.area a ON (a.id_area=s.id_area) ");
            query.append("WHERE d.estado='1' ");
            query.append("ORDER BY d.id_adset_name ASC ");
		
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	AdSetName obj=new AdSetName();
            	obj.setId(rs.getInt("id_adset_name"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	obj.setDescripcion(rs.getString("descripcion"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setSubArea(rs.getString("sub_area"));
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setArea(rs.getString("area"));
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto"));
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setEstado(rs.getString("estado"));
            	obj.setEsIndividual(rs.getInt("es_individual"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}


	/**
	 * mantenimiento de perfiles
	 */
	
	public Integer actualizaPerfil(Integer id,Integer idUsuario,Integer idPerfil,Integer idCategoria,Integer idSubCategoria,
			Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.usuario_perfil ");
            query.append("SET id_usuario = ? ");
            query.append(",id_modulo = 2 ");
            query.append(",id_perfil = ? ");
            query.append(",id_categoria = ? ");
            query.append(",id_sub_categoria = ? ");
            query.append(",estado = ? ");
            query.append("WHERE id_usuario_perfil=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idUsuario);
            ps.setInt(2, idPerfil);
            ps.setInt(3, idCategoria);
            ps.setInt(4, idSubCategoria);
            ps.setString(5, String.valueOf(Constante.ESTADO_ACTIVO));
            ps.setInt(6, id);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	
	public Integer registraPerfil(Integer idUsuario,Integer idPerfil,Integer idCategoria,Integer idSubCategoria,
			Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO leads.usuario_perfil(id_usuario,id_modulo,id_perfil,id_categoria,id_sub_categoria,estado)VALUES (?,2,?,?,?,?) ");
            String ids[] = new String[] { "id_adset_name" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setInt(1, idUsuario);
            ps.setInt(2, idPerfil);
            ps.setInt(3, idCategoria);
            ps.setInt(4, idSubCategoria);
            ps.setString(5, String.valueOf(Constante.ESTADO_ACTIVO));
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return pk;
	}
	
	public Integer eliminarPerfil(Integer id,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE leads.usuario_perfil SET estado = ?  WHERE id_usuario_perfil=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, String.valueOf(Constante.ESTADO_ELIMINADO));
            ps.setInt(2, id);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	public List<PerfilLead> listaPerfil(){
		List<PerfilLead> lista = new ArrayList<PerfilLead>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT up.id_usuario_perfil ");
            query.append(",up.id_usuario ");
            query.append(",u.username	 ");
            query.append(",u.apellidos_nombres ");
            query.append(",up.id_perfil ");
            query.append(",(CASE  ");
            query.append("WHEN up.id_perfil=1 THEN 'ADMINISTRA'  ");
            query.append("WHEN up.id_perfil=2 THEN 'ASIGNA'  ");
            query.append("WHEN up.id_perfil=3 THEN 'SEGUIMIENTO'  ");
            query.append("WHEN up.id_perfil=4 THEN 'CONSULTA' "); 
            query.append("WHEN up.id_perfil=5 THEN 'SEGUIMIENTO-CORREOS' ");
            query.append("WHEN up.id_perfil=6 THEN 'ASIGNA-CORREOS' ");
            query.append("END) perfil ");
            query.append(",a.id_area ");
            query.append(",a.denominacion area ");
            query.append(",b.id_sub_area ");
            query.append(",b.denominacion sub_area ");
            query.append("FROM leads.usuario_perfil up ");
            query.append("INNER JOIN dbo.usuario u on (up.id_usuario=u.id_usuario) ");
            query.append("INNER JOIN dbo.area a on (a.id_area=up.id_categoria) ");
            query.append("INNER JOIN leads.sub_area b on (b.id_sub_area=up.id_sub_categoria) ");
            query.append("WHERE up.estado='1'  ");
            query.append("ORDER BY a.denominacion,b.denominacion,up.id_perfil,u.apellidos_nombres ASC ");
		
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	PerfilLead obj=new PerfilLead();
            	obj.setId(rs.getInt("id_usuario_perfil"));
            	obj.setIdUsuario(rs.getInt("id_usuario"));
            	obj.setUsername(rs.getString("username"));;
            	obj.setApellidosNombres(rs.getString("apellidos_nombres"));;
            	obj.setIdPerfil(rs.getInt("id_perfil"));
            	obj.setPerfil(rs.getString("perfil"));;
            	obj.setIdArea(rs.getInt("id_area"));
            	obj.setIdSubArea(rs.getInt("id_sub_area"));
            	obj.setArea(rs.getString("area"));;
            	obj.setSubArea(rs.getString("sub_area"));;
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	public List<Value> listUsuario(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT e.id_usuario id,upper(e.apellidos_nombres) denominacion FROM dbo.usuario e "); 
            query.append("where e.id_area in (1,6,9,16,17,12)  ");
            query.append("order by e.apellidos_nombres asc ");
		
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}

	public List<ReporteGeneral> listaReporteGeneral(Integer idArea,Integer idSubArea,Integer idAdsetName,
			String fechaAdsetIni,String fechaAdsetFin,
			String fechaContactoIni,String fechaContactoFin,
			String fechaProximaIni,String fechaProximaFin,
			String fechaCargaIni,String fechaCargaFin,
			Integer idEjecutivo,Integer idEstado){
		List<ReporteGeneral> lista = new ArrayList<ReporteGeneral>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_lead ");
            query.append(",adset_name ");
            query.append(",area ");
            query.append(",sub_area ");
            query.append(",precio_pax_adulto ");
            query.append(",precio_pax_ninyo ");
            query.append(",fecha_adset ");
            query.append(",pax_adulto ");
            query.append(",pax_ninyo ");
            query.append(",nombre_cliente ");
            query.append(",email_cliente ");
            query.append(",telefono_cliente ");
            query.append(",fecha_carga ");
            query.append(",total_paq_adulto ");
            query.append(",total_paq_ninyo ");
            query.append(",total_reserva ");
            query.append(",cotizado ");
            query.append(",fecha_contacto ");
            query.append(",canal ");
            query.append(",reserva ");
            query.append(",motivo ");
            query.append(",pax_final ");
            query.append(",revenue ");
            query.append(",servicio_final ");
            query.append(",ejecutivo ");
            query.append(",observacion ");
            query.append(",fecha_proxima ");
            query.append(",seq_seguimiento ");
            query.append(",id_estado_seguimiento ");
            query.append("FROM dbo.view_reporte_general_leads WHERE 1=1 ");
            if(idArea!=null&&idArea>0){
            	query.append(" AND id_area=? ");	
            	if(idSubArea!=null&&idSubArea>0){
            		query.append(" AND id_sub_area=? ");
            	}
            }
            if(idAdsetName!=null&&idAdsetName>0){
            	query.append(" AND id_adset_name=? ");
            }
            if(fechaAdsetIni!=null&&fechaAdsetIni.trim().length()>0&&
            		fechaAdsetFin!=null&&fechaAdsetFin.trim().length()>0){
            	query.append(" AND f_fecha_adset between ? and ? ");
            }
            if(fechaContactoIni!=null&&fechaContactoIni.trim().length()>0&&
            		fechaContactoFin!=null&&fechaContactoFin.trim().length()>0){
            	query.append(" AND f_fecha_contacto between ? and ? ");
            }
            if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
            		fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
            	query.append(" AND f_fecha_proxima between ? and ? ");
            }
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
            		fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
            	query.append(" AND f_fecha_carga between ? and ? ");
            }
            if(idEjecutivo!=null&&idEjecutivo>0){
            	query.append(" AND id_ejecutiva = ? ");
            }
            
            if(idEstado<9){
        		query.append(" AND id_estado_seguimiento=? ");
        	}
            
            query.append(" ORDER BY id_lead DESC ");
            
            Integer index=1; 
            ps = conn.prepareStatement(query.toString());
            if(idArea!=null&&idArea>0){
            	ps.setInt(index++, idArea);	
            	if(idSubArea!=null&&idSubArea>0){
            		ps.setInt(index++, idSubArea);
            	}
            }
            if(idAdsetName!=null&&idAdsetName>0){
            	ps.setInt(index++, idAdsetName);
            }
            if(fechaAdsetIni!=null&&fechaAdsetIni.trim().length()>0&&
            		fechaAdsetFin!=null&&fechaAdsetFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaAdsetIni));
            	ps.setDate(index++, Util.stringToDate(fechaAdsetFin));
            }
            if(fechaContactoIni!=null&&fechaContactoIni.trim().length()>0&&
            		fechaContactoFin!=null&&fechaContactoFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaContactoIni));
            	ps.setDate(index++, Util.stringToDate(fechaContactoFin));
            }
            if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
            		fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaProximaIni));
            	ps.setDate(index++, Util.stringToDate(fechaProximaFin));
            }
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
            		fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaCargaIni));
            	ps.setDate(index++, Util.stringToDate(fechaCargaFin));
            }
            
            if(idEjecutivo!=null&&idEjecutivo>0){
            	ps.setInt(index++, idEjecutivo);
            }
            
            if(idEstado<9){
        		ps.setInt(index++, idEstado);
        	}
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ReporteGeneral obj=new ReporteGeneral();
            	
            	obj.setIdLead(rs.getInt("id_lead")); 
            	obj.setAdsetName(rs.getString("adset_name")); 
            	obj.setArea(rs.getString("area"));
            	obj.setSubArea(rs.getString("sub_area")); 
            	obj.setPrecioPaxAdulto(rs.getDouble("precio_pax_adulto")); 
            	obj.setPrecioPaxNinyo(rs.getDouble("precio_pax_ninyo"));
            	obj.setFechaAdset(rs.getString("fecha_adset"));
            	obj.setPaxAdulto(rs.getInt("pax_adulto"));
            	obj.setPaxNinyo(rs.getInt("pax_ninyo"));
            	obj.setNombreCliente(rs.getString("nombre_cliente")); 
            	obj.setEmailCliente(rs.getString("email_cliente"));
            	obj.setTelefonoCliente(rs.getString("telefono_cliente")); 
            	obj.setFechaCarga(rs.getString("fecha_carga"));
            	obj.setTotalPaqAdulto(rs.getDouble("total_paq_adulto")); 
            	obj.setTotalPaqNinyo(rs.getDouble("total_paq_ninyo"));
            	obj.setTotalReserva(rs.getDouble("total_reserva"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto")); 
            	obj.setCanal(rs.getString("canal"));
            	obj.setReserva(rs.getString("reserva")); 
            	obj.setMotivo(rs.getString("motivo"));
                obj.setPaxFinal(rs.getInt("pax_final")); 
                obj.setRevenue(rs.getDouble("revenue"));
                obj.setServicioFinal(rs.getString("servicio_final")); 
                obj.setEjecutivo(rs.getString("ejecutivo"));
                obj.setObservacion(rs.getString("observacion")); 
                obj.setFechaProxima(rs.getString("fecha_proxima")); 
                obj.setSeqSeguimiento(rs.getInt("seq_seguimiento"));
                obj.setIdEstadoSeguimiento(rs.getInt("id_estado_seguimiento"));
                
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}

	/*
	 * perfil / usuarios
	if(idPerfil==1)//<option value="1">ADMINISTRA</option>
	{
		sqlWhere.append(" AND p.id_perfil IN (6,2) ");//<option value="6">ASIGNA-CORREO</option> // <option value="2">ASIGNA</option> 
	}else if((idPerfil==6)||(idPerfil==2)){
		sqlWhere.append(" AND p.id_perfil IN (5,3) AND p.id_categoria="+idArea+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
	}else if((idPerfil==5)||(idPerfil==3)){
		sqlWhere.append(" AND p.id_perfil IN (5,3) AND e.id_usuario="+idUsuario+"  ");//<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
	}else{
		sqlWhere.append(" AND 1!=1 ");
	}
	*/
	
	//lista de correos
	public String sqlWherefromProfileEmail(Integer idLeadPerfil,Integer idUsuario,String ids){
		StringBuilder sqlWhere = new StringBuilder();
		
		if(idLeadPerfil==5||idLeadPerfil==3){////<option value="5">SEGUIMIENTO-CORREO</option>  // <option value="3">SEGUIMIENTO</option>
			sqlWhere.append(" AND e.id_ejecutiva=").append(String.valueOf(idUsuario));
		}else if(idLeadPerfil==6||idLeadPerfil==2) {/////<option value="6">ASIGNA-CORREO</option> // <option value="2">ASIGNA</option>
			//sqlWhere.append(" AND e.id_ejecutiva IN ("+idUsuario+","+ids+")");
			sqlWhere.append(" AND 1=1 ");
		}else{
			sqlWhere.append(" AND 1=1 ");
		}
		return sqlWhere.toString();
		
	}
	
	public List<Correo> listCorreos(String fechaIni,String fechaFin,Integer idEstado,Integer pIdEjecutivo,String ids,
			String fechaProximaIni,String fechaProximaFin,Integer idSeguimiento,
			Integer idArea,Integer idAdsetName,Integer idReserva,Integer idMotivo,Integer cantSeg,Integer idCotizado,
			Usuario pUser){
		List<Correo> lista = new ArrayList<Correo>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
        	//update dbo.email set dSentOn=convert(date,SUBSTRING(SentOn,0,11),103),dReceivedTime=convert(date,SUBSTRING(ReceivedTime,0,11),103)
            StringBuilder query = new StringBuilder();
            query.append("SELECT e.id_email,e.toEmail,e.SenderEmailAddress,e.SubjectEmail,e.BodyEmail,dbo.getDateTimeAMPM(e.dSentOn) sSentOn,e.SentOn,e.ReceivedTime ");
            query.append(",isnull(u.username,'') username,isnull(u.id_usuario,0) id_usuario ");
            query.append(",isnull(a.denominacion,'') area,isnull(a.id_area,0) id_area ");
            query.append(",(SELECT count(1) FROM dbo.email_seguimiento s where s.id_email=e.id_email) seguimiento ");
            query.append(",convert(varchar(20),ls.fecha_proxima,103) fecha_proxima ");
            query.append(",dbo.getDateTimeAMPM(e.audit_fechahora) fecha_asigna_ejecutivo,dbo.getDateTimeAMPM(e.fecha_asigna_area) fecha_asigna_area ");
            query.append(",n.denominacion servicio,dbo.getReserva(ls.id_reserva) reserva, dbo.getMotivo(ls.id_motivo) motivo,ls.seq_seguimiento,dbo.getCotiza(ls.id_cotizado) cotizado,convert(varchar(20),ls.fecha_contacto,103) fecha_contacto,dbo.getCanal(ls.id_canal) canal ");
            query.append(",ISNULL(ls.pax_final,0) pax_final,ISNULL(ls.revenue,0) revenue ");
            query.append(",dbo.getEstadoCorreo(e.id_email) estado_email ");
            query.append("FROM dbo.email e  ");
            query.append("left JOIN dbo.usuario u ON (e.id_ejecutiva=u.id_usuario) ");
            query.append("left JOIN dbo.area a ON (a.id_area=e.id_area) ");
            query.append("left join dbo.email_seguimiento ls on (ls.id_email=e.id_email and ls.estado='1') ");
            query.append("left join leads.adset_name n on (n.id_adset_name=ls.id_adset_name_final) ");
            query.append("WHERE e.estado ='1' ");
            query.append(sqlWherefromProfileEmail(pUser.getIdPerfilLead(), pUser.getUsuarioId(),ids));
            
            if(fechaIni!=null&&fechaIni.trim().length()>0&&
      			   fechaFin!=null&&fechaFin.trim().length()>0){
      	    	query.append(" AND e.dSentOn between ? and ? ");
      	    }
 			
            if(idEstado!=null && idEstado<9){
             	if(idEstado==0){//NO ASIGNADO
             		query.append("and ISNULL(e.id_ejecutiva,0)=? ");	
             	}else if(idEstado==1){//SI ASIGNADO
             		query.append("and ISNULL(e.id_ejecutiva,0)>=? ");
             	}
            }
            
            if(pIdEjecutivo!=null && pIdEjecutivo>0){
             	query.append("and ISNULL(e.id_ejecutiva,0)=? ");	
            }
            
            if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
        			   fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
        	    query.append("and ls.fecha_proxima between ? and ? ");
        	}
 			

            if(idSeguimiento!=null && idSeguimiento<9){
             	if(idSeguimiento==0){//NO ASIGNADO
             		query.append("and ISNULL(ls.id_email_seguimiento,0)=? ");	
             	}else if(idSeguimiento==1){//SI ASIGNADO
             		query.append("and ISNULL(ls.id_email_seguimiento,0)>=? ");
             	}
             }
            //nuevos filtros
            if(idArea!=null && idArea>0){
             	query.append("and ISNULL(e.id_area,0)=? ");	
            }
            if(idAdsetName!=null && idAdsetName>0){
             	query.append("and ISNULL(ls.id_adset_name_final,0)=? ");	
            }
            if(idReserva!=null && idReserva>0){
             	query.append("and ISNULL(ls.id_reserva,0)=? ");	
            }
            if(idMotivo!=null && idMotivo>0){
             	query.append("and ISNULL(ls.id_motivo,0)=? ");	
            }
            if(cantSeg!=null && cantSeg<99){
             	query.append("and ISNULL(ls.seq_seguimiento,0)=? ");	
            }
            
            if(idCotizado!=null && idCotizado>0){
            	query.append("and ISNULL(ls.id_cotizado,0)=? ");		   
     	    }

            
            query.append("ORDER BY e.id_email DESC  ");

            ps = conn.prepareStatement(query.toString());
            
   	    	Integer index=1;
   	    	if(fechaIni!=null&&fechaIni.trim().length()>0&&
   	    	     			   fechaFin!=null&&fechaFin.trim().length()>0){
     	    	ps.setDate(index++, Util.stringToDate(fechaIni));
     	    	ps.setDate(index++, Util.stringToDate(fechaFin));
     	    }
   	    	if(idEstado!=null && idEstado<9){
            	if(idEstado==0){//NO ASIGNADO
            		ps.setInt(index++, idEstado);	
            	}else if(idEstado==1){//SI ASIGNADO
            		ps.setInt(index++, idEstado);
            	}
     	    }
            
   	    	if(pIdEjecutivo!=null && pIdEjecutivo>0){
           		ps.setInt(index++, pIdEjecutivo);	
     	    }
   	    	
   	    	if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
        			   fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
        	    	ps.setDate(index++, Util.stringToDate(fechaProximaIni));
        	    	ps.setDate(index++, Util.stringToDate(fechaProximaFin));
        	}

   	    	if(idSeguimiento!=null && idSeguimiento<9){
             	if(idSeguimiento==0){//NO ASIGNADO
             		ps.setInt(index++, idSeguimiento);	
             	}else if(idSeguimiento==1){//SI ASIGNADO
             		ps.setInt(index++, idSeguimiento);
             	}
      	    }
   	    	//nuevos filtros
            if(idArea!=null && idArea>0){
            	ps.setInt(index++, idArea);
            }
            if(idAdsetName!=null && idAdsetName>0){
            	ps.setInt(index++, idAdsetName);	
            }
            if(idReserva!=null && idReserva>0){
            	ps.setInt(index++, idReserva);	
            }
            if(idMotivo!=null && idMotivo>0){
            	ps.setInt(index++, idMotivo);	
            }
            if(cantSeg!=null && cantSeg<99){
            	ps.setInt(index++, cantSeg);	
            }
            if(idCotizado!=null && idCotizado>0){
            	ps.setInt(index++, idCotizado);		   
     	    }
            
   	    	rs = ps.executeQuery();
            
            while(rs.next()){
            	Correo obj=new Correo();
            	obj.setToEmail(rs.getString("toEmail"));
            	obj.setSenderEmailAddress(rs.getString("SenderEmailAddress"));
            	obj.setSubjectEmail(rs.getString("SubjectEmail"));
            	obj.setBodyEmail(rs.getString("BodyEmail"));
            	obj.setdSentOn(rs.getString("sSentOn"));
            	obj.setSentOn(rs.getString("SentOn"));
            	obj.setReceivedTime(rs.getString("ReceivedTime"));
            	obj.setIdEmail(rs.getInt("id_email"));
            	
            	obj.setIdEjecutivo(rs.getInt("id_usuario"));
            	obj.setEjecutivo(rs.getString("username"));
            	obj.setSeguimiento(rs.getInt("seguimiento"));
            	
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	
            	obj.setFechaAsignaEjecutivo(rs.getString("fecha_asigna_ejecutivo"));
            	obj.setFechaAsignaArea(rs.getString("fecha_asigna_area"));
            	//--
            	obj.setServicio(rs.getString("servicio"));
            	obj.setReserva(rs.getString("reserva"));
            	obj.setMotivo(rs.getString("motivo"));
            	obj.setSeqSeguimiento(rs.getInt("seq_seguimiento"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	obj.setArea(rs.getString("area"));
            	//
            	obj.setPaxFinal(rs.getInt("pax_final"));
            	obj.setRevenue(rs.getDouble("revenue"));
            	//
            	obj.setEstadoEmail(rs.getInt("estado_email"));
            	
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return lista;
	}

	/**
	 * 
	 */
	public Integer guardarSeguimientoCorreo(Integer idEjecutivo,Integer idEmail,Integer idCotizado,Integer idCanal,
			Integer idReserva,Integer idMotivo,Integer paxFinal,Double revenue,Integer idAdSetNameFinal,String observacion,String fechaProxima,Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO dbo.email_seguimiento(id_cotizado,fecha_contacto,id_canal,id_reserva,id_motivo,pax_final,revenue,id_adset_name_final,id_usuario,id_email,observacion,fecha_proxima,seq_seguimiento,estado) ");
            query.append("VALUES(?,getdate(),?,?,?,?,?,?,?,?,?,?,?,?) ");
            String ids[] = new String[] { "id_email_seguimiento" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setInt(1, idCotizado);
            ps.setInt(2, idCanal);
            ps.setInt(3, idReserva);
            ps.setInt(4, idMotivo);
            ps.setInt(5, paxFinal==null?0:paxFinal);
            ps.setDouble(6, revenue==null?0.0:revenue);
            ps.setInt(7, idAdSetNameFinal);
            ps.setInt(8, idEjecutivo);
            ps.setInt(9, idEmail);
            ps.setString(10, observacion==null?"":observacion);
            
            if(fechaProxima.trim().length()>0){
            	ps.setDate(11, Util.stringToDate(fechaProxima));	
            }else{
            	ps.setNull(11, java.sql.Types.DATE);
            }
            
            ps.setInt(12, Constante.ESTADO_ACTIVO);//1
            ps.setString(13, String.valueOf(Constante.ESTADO_ACTIVO));//'1'
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
		return pk;
	}
	
	public Integer actualiaEstadoSeguimientoCorreo(Integer idLeadSeguimiento){
		CallableStatement cs = null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("{ call dbo.estado_email_seguimiento(?) }");
            
            cs = conn.prepareCall(query.toString());
            cs.setInt(1, idLeadSeguimiento);
            
            cs.execute();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (cs != null) {try {cs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return 1;
	}
	
	public List<LeadSeguimiento> listCorreoSeguimientoByIdEmail(Integer idEmail){
		List<LeadSeguimiento> lista = new ArrayList<LeadSeguimiento>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_email_seguimiento "); 
            query.append(",(case when id_cotizado=1 then 'SI' when id_cotizado=2 then 'NO' when id_cotizado=3 then 'CERRADO' END) cotizado ");  
            query.append(",convert(varchar(20),fecha_contacto,103) fecha_contacto   ");
            query.append(",(case when id_canal=1 then 'CORREO' WHEN id_canal=2 THEN 'LLAMADA' WHEN id_canal=3 THEN 'PRESENCIAL'  WHEN id_canal=4 THEN 'WHATSAPP'  ELSE 'NINGUNO' END) canal ");  
            query.append(",(case when id_reserva=1 then 'SI' WHEN id_reserva=3 then 'Seg' ELSE 'NO' END) reserva   ");
            query.append(",(CASE    ");
            query.append("WHEN id_motivo=1 then 'NO CONTESTA' ");  
            query.append("WHEN id_motivo=2 then 'NO ESTA INTERESADO' ");  
            query.append("WHEN id_motivo=3 then 'POR CONFIRMAR'   ");
            query.append("WHEN id_motivo=4 then 'NUMERO INVALIDO'   ");
            query.append("WHEN id_motivo=5 then 'VOLVER A CONTACTAR'   ");
            query.append("WHEN id_motivo=6 then 'TOMO OTRA OPCION'   ");
            query.append("WHEN id_motivo=7 then 'SI RESERVA'   ");
            query.append("ELSE 'NINGUNO'   ");
            query.append("END) motivo   ");
            query.append(",pax_final   ");
            query.append(",revenue   ");
            query.append(",( SELECT denominacion  FROM leads.adset_name where id_adset_name=s.id_adset_name_final ) servicio_final "); 
            query.append(",e.id_usuario   ");
            query.append(",s.id_email ");
            query.append(",e.username ejecutivo ");  
            query.append(",s.observacion  ");
            query.append(",convert(varchar(20),s.fecha_proxima,103) fecha_proxima "); 
            query.append(",s.seq_seguimiento  ");
            query.append(",s.estado ");
            query.append("FROM dbo.email_seguimiento s ");  
            query.append("inner join dbo.usuario e on (e.id_usuario=s.id_usuario) ");  
            query.append("where s.id_email=? ORDER BY id_email_seguimiento ASC    ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idEmail);
            
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	LeadSeguimiento obj=new LeadSeguimiento();
            	obj.setIdLead(rs.getInt("id_email"));
            	obj.setIdLeadSeguimiento(rs.getInt("id_email_seguimiento"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	obj.setReserva(rs.getString("reserva"));
            	obj.setMotivo(rs.getString("motivo"));
            	obj.setPaxFinal(rs.getString("pax_final"));
            	obj.setRevenue(rs.getString("revenue"));
            	obj.setServicioFinal(rs.getString("servicio_final"));
            	obj.setIdUsuario(rs.getInt("id_usuario"));
            	obj.setIdLead(rs.getInt("id_email"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	//--
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	obj.setObservacion(rs.getString("observacion"));
            	obj.setSecuencia(rs.getInt("seq_seguimiento"));
            	obj.setEstado(rs.getString("estado"));
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public Integer eliminarCorreo(Integer id,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE dbo.email SET estado = ?  WHERE id_email=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, String.valueOf(Constante.ESTADO_ELIMINADO));
            ps.setInt(2, id);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	public LeadSeguimiento getCorreoSeguimiento(Integer idLeadSeguimiento){
		LeadSeguimiento obj = null; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT s.id_email_seguimiento "); 
            query.append(",(case when id_cotizado=1 then 'SI' when id_cotizado=2 then 'NO' when id_cotizado=3 then 'CERRADO' END) cotizado ");  
            query.append(",convert(varchar(20),fecha_contacto,103) fecha_contacto   ");
            query.append(",(case when id_canal=1 then 'CORREO' WHEN id_canal=2 THEN 'LLAMADA' WHEN id_canal=3 THEN 'PRESENCIAL'  WHEN id_canal=4 THEN 'WHATSAPP'  ELSE 'NINGUNO' END) canal ");  
            query.append(",(case when id_reserva=1 then 'SI' WHEN id_reserva=3 then 'Seg' ELSE 'NO' END) reserva   ");
            query.append(",(CASE    ");
            query.append("WHEN id_motivo=1 then 'NO CONTESTA' ");  
            query.append("WHEN id_motivo=2 then 'NO ESTA INTERESADO' ");  
            query.append("WHEN id_motivo=3 then 'POR CONFIRMAR'   ");
            query.append("WHEN id_motivo=4 then 'NUMERO INVALIDO'   ");
            query.append("WHEN id_motivo=5 then 'VOLVER A CONTACTAR'   ");
            query.append("WHEN id_motivo=6 then 'TOMO OTRA OPCION'   ");
            query.append("WHEN id_motivo=7 then 'SI RESERVA'   ");
            query.append("ELSE 'NINGUNO'   ");
            query.append("END) motivo   ");
            query.append(",pax_final   ");
            query.append(",revenue   ");
            query.append(",( SELECT denominacion  FROM leads.adset_name where id_adset_name=s.id_adset_name_final ) servicio_final "); 
            query.append(",e.id_usuario   ");
            query.append(",s.id_email ");
            query.append(",e.username ejecutivo ");  
            query.append(",s.observacion  ");
            query.append(",convert(varchar(20),s.fecha_proxima,103) fecha_proxima "); 
            query.append(",s.seq_seguimiento  ");
            query.append(",s.id_cotizado ");
            query.append(",s.id_canal ");
            query.append(",s.id_motivo ");
            query.append(",s.id_reserva ");
            query.append(",s.id_adset_name_final ");
            query.append("FROM dbo.email_seguimiento s ");  
            query.append("inner join dbo.usuario e on (e.id_usuario=s.id_usuario) ");  
            query.append("where s.id_email_seguimiento=? ORDER BY id_email_seguimiento DESC    ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idLeadSeguimiento);
            
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	obj=new LeadSeguimiento();
            	obj.setIdLead(rs.getInt("id_email"));
            	obj.setIdLeadSeguimiento(rs.getInt("id_email_seguimiento"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	obj.setReserva(rs.getString("reserva"));
            	obj.setMotivo(rs.getString("motivo"));
            	obj.setPaxFinal(rs.getString("pax_final"));
            	obj.setRevenue(rs.getString("revenue"));
            	obj.setServicioFinal(rs.getString("servicio_final"));
            	obj.setIdUsuario(rs.getInt("id_usuario"));
            	obj.setIdLead(rs.getInt("id_email"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	//--
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	obj.setObservacion(rs.getString("observacion"));
            	obj.setSecuencia(rs.getInt("seq_seguimiento"));
	            	obj.setIdCotizado(rs.getInt("id_cotizado"));
	        		obj.setIdCanal(rs.getInt("id_canal"));
	        		obj.setIdReserva(rs.getInt("id_reserva"));
	        		obj.setIdMotivo(rs.getInt("id_motivo"));
	        		obj.setIdServicioFinal(rs.getInt("id_adset_name_final"));
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return obj;
	}
	
	public LeadSeguimiento getLeadSeguimiento(Integer idLeadSeguimiento){
		LeadSeguimiento obj = null; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_lead_seguimiento ");
            query.append(",(case when id_cotizado=1 then 'SI' when id_cotizado=2 then 'NO' when id_cotizado=3 then 'CERRADO' END) cotizado "); 
            query.append(",convert(varchar(20),fecha_contacto,103) fecha_contacto  ");
            query.append(",(case when id_canal=1 then 'CORREO' WHEN id_canal=2 THEN 'LLAMADA' WHEN id_canal=3 THEN 'PRESENCIAL' WHEN id_canal=4 THEN 'WHATSAPP' ELSE 'NINGUNO' END) canal "); 
            query.append(",(case when id_reserva=1 then 'SI' WHEN id_reserva=3 then 'Seg' ELSE 'NO' END) reserva  ");
            query.append(",(CASE   ");
            query.append("WHEN id_motivo=1 then 'NO CONTESTA' "); 
            query.append("WHEN id_motivo=2 then 'NO ESTA INTERESADO' "); 
            query.append("WHEN id_motivo=3 then 'POR CONFIRMAR'  ");
            query.append("WHEN id_motivo=4 then 'NUMERO INVALIDO'  ");
            query.append("WHEN id_motivo=5 then 'VOLVER A CONTACTAR'  ");
            query.append("WHEN id_motivo=6 then 'TOMO OTRA OPCION'  ");
            query.append("WHEN id_motivo=7 then 'SI RESERVA'  ");
            query.append("ELSE 'NINGUNO'  ");
            query.append("END) motivo  ");
            query.append(",pax_final  ");
            query.append(",revenue  ");
            query.append(",( SELECT denominacion  FROM leads.adset_name where id_adset_name=s.id_adset_name_final ) servicio_final ");
            query.append(",e.id_usuario  ");
            query.append(",s.id_lead  ");
            query.append(",e.username ejecutivo "); 
            query.append(",s.observacion ");
            query.append(",convert(varchar(20),s.fecha_proxima,103) fecha_proxima ");
            query.append(",s.seq_seguimiento ");
            query.append(",s.id_cotizado ");
            query.append(",s.id_canal ");
            query.append(",s.id_motivo ");
            query.append(",s.id_reserva ");
            query.append(",s.id_adset_name_final ");
            query.append(",s.hora ");
            query.append("FROM leads.lead_seguimiento s  ");
            query.append("inner join dbo.usuario e on (e.id_usuario=s.id_usuario) "); 
            query.append("where s.id_lead_seguimiento=? ORDER BY id_lead_seguimiento ASC ");
			
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idLeadSeguimiento);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	obj=new LeadSeguimiento();	
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setIdLeadSeguimiento(rs.getInt("id_lead_seguimiento"));
            		obj.setCotizado(rs.getString("cotizado"));
            		obj.setCanal(rs.getString("canal"));
            		obj.setReserva(rs.getString("reserva"));
            		obj.setMotivo(rs.getString("motivo"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));	
            	obj.setPaxFinal(rs.getString("pax_final"));
            	obj.setRevenue(rs.getString("revenue"));
            	obj.setServicioFinal(rs.getString("servicio_final"));
            	obj.setIdUsuario(rs.getInt("id_usuario"));
            	obj.setIdLead(rs.getInt("id_lead"));
            	obj.setEjecutivo(rs.getString("ejecutivo"));
            	obj.setFechaProxima(rs.getString("fecha_proxima"));
            	obj.setObservacion(rs.getString("observacion"));
            	obj.setSecuencia(rs.getInt("seq_seguimiento"));
	            	obj.setIdCotizado(rs.getInt("id_cotizado"));
	        		obj.setIdCanal(rs.getInt("id_canal"));
	        		obj.setIdReserva(rs.getInt("id_reserva"));
	        		obj.setIdMotivo(rs.getInt("id_motivo"));
	        		obj.setIdServicioFinal(rs.getInt("id_adset_name_final"));
	        		obj.setIdHora(rs.getInt("hora"));
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return obj;
	}
	
	
	public Integer actualizarSeguimientoCorreo(Integer idLeadSeguimiento,Integer idLead,Integer idCotizado,Integer idCanal,
			Integer idReserva,Integer idMotivo,Integer paxFinal,Double revenue,Integer idAdSetNameFinal,String observacion,String fechaProxima,Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE dbo.email_seguimiento ");
            query.append("SET id_cotizado = ? ");
            query.append(",id_canal = ? ");
            query.append(",id_reserva = ? ");
            query.append(",id_motivo = ? ");
            query.append(",pax_final = ? ");
            query.append(",revenue = ? ");
            query.append(",observacion = ? ");
            query.append(",id_adset_name_final = ? ");
            query.append(",fecha_proxima = ? ");
            query.append("WHERE id_email_seguimiento=? and id_email=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idCotizado);
            ps.setInt(2, idCanal);
            ps.setInt(3, idReserva);
            ps.setInt(4, idMotivo);
            ps.setInt(5, paxFinal==null?0:paxFinal);
            ps.setDouble(6, revenue==null?0.0:revenue);
            ps.setString(7, observacion==null?"":observacion);
            ps.setInt(8, idAdSetNameFinal);
            if(fechaProxima.trim().length()>0){
            	ps.setDate(9, Util.stringToDate(fechaProxima));	
            }else{
            	ps.setNull(9, java.sql.Types.DATE);
            }
            ps.setInt(10, idLeadSeguimiento);
            ps.setInt(11, idLead);
            //--
            if(ps.executeUpdate()>0){
            	pk =idLeadSeguimiento;
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
		return pk;
	}
	
	public List<Integer> getList(String idLeads){
		List<Integer> iList=new LinkedList<>();
		if(idLeads!=null&&idLeads.length()>0){
			idLeads=idLeads.substring(1, idLeads.length());
			String[] myData = idLeads.split(",");
			for (String s: myData) {
			    iList.add(Integer.valueOf(s));
			}	
		}
		return iList;
	}
	
	public Integer guardarMailMasivo(String mailIdLeads,String observacion,Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO leads.lead_seguimiento(id_cotizado,fecha_contacto,id_canal,id_reserva,id_motivo,pax_final,revenue,id_adset_name_final,id_usuario,id_lead,observacion,fecha_proxima,seq_seguimiento,estado) ");
            query.append("VALUES(?,getdate(),?,?,?,?,?,?,?,?,?,?,?,?) ");
            
            List<Integer> l=getList(mailIdLeads);
            
            for (Iterator<Integer> it = l.iterator(); it.hasNext(); ) {
    		    if (it.hasNext()) {
    		    	String ids[] = new String[] { "id_lead_seguimiento" };
    		    	
    		    	ps = conn.prepareStatement(query.toString(),ids);
    	            ps.setInt(1, 1);//SI
    	            ps.setInt(2, 1);//Correo
    	            ps.setInt(3, 2);//NO
    	            ps.setInt(4, 5);//Volver a contactar
    	            ps.setInt(5, 0);//paxFinal
    	            ps.setDouble(6, 0.0);//revenue
    	            ps.setInt(7, 0);//idAdSetNameFinal
    	            ps.setInt(8, pUser.getUsuarioId());
    	            ps.setInt(9, it.next());
    	            ps.setString(10, observacion==null?"":observacion);
    	            ps.setNull(11, java.sql.Types.DATE);
    	            ps.setInt(12, Constante.ESTADO_ACTIVO);//1
    	            ps.setString(13, String.valueOf(Constante.ESTADO_ACTIVO));//'1'
    	            if(ps.executeUpdate()>0){
    	            	rs = ps.getGeneratedKeys();
    	                if ( rs.next() ) {
    	                    pk = rs.getInt(1);
    	                    actualiaEstadoSeguimiento(pk);
    	                }
    	            }
    		    }
    		}
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
		return pk;
	}
	
	public Integer eliminarMailMasivo(String mailIdLeads,Usuario pUser){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=Constante.ID_INICIAL;
        try {
        	
        	mailIdLeads=mailIdLeads.substring(1, mailIdLeads.length());
        	
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE dbo.email SET estado = ?  WHERE id_email IN ("+mailIdLeads+") ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, Constante.ESTADO_ELIMINADO);
    	    ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
		return pk;
	}
	
	/***
	 * 
	 * seguimiento de correos
	 */
	
	public List<ReporteCorreos> listaReporteCorreos(Integer idArea,
			String fechaProximaIni,String fechaProximaFin,
			String fechaCargaIni,String fechaCargaFin,
			Integer idEjecutivo,Integer idEstado){
		List<ReporteCorreos> lista = new ArrayList<ReporteCorreos>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_email ");
            query.append(",SenderEmailAddress ");
            query.append(",SubjectEmail ");
            query.append(",dSentOn ");
            query.append(",cotizado ");
            query.append(",fecha_contacto ");
            query.append(",canal ");
            query.append(",reserva ");
            query.append(",motivo ");
            query.append(",pax_final ");
            query.append(",revenue ");
            query.append(",ejecutivo ");
            query.append(",observacion ");
            query.append(",fecha_proxima ");
            query.append(",seq_seguimiento ");
            query.append(",fecha_contacto ");
            query.append(",f_fecha_proxima ");
            query.append(",posee_seguimiento ");
            query.append(",adset_name ");
            query.append(",area ");
            query.append(",id_estado_seguimiento ");
            query.append("FROM dbo.view_reporte_general_emails WHERE 1=1 ");
            if(idArea!=null&&idArea>0){
            	query.append(" AND id_area=? ");	
            }
            if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
            		fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
            	query.append(" AND f_fecha_proxima between ? and ? ");
            }
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
            		fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
            	query.append(" AND f_fecha_carga between ? and ? ");
            }
            if(idEjecutivo!=null&&idEjecutivo>0){
            	query.append(" AND id_ejecutiva = ? ");
            }
            
            if(idEstado<9){
        		query.append(" AND id_estado_seguimiento=? ");
        	}
            
            query.append(" ORDER BY id_email DESC ");
            
            Integer index=1; 
            ps = conn.prepareStatement(query.toString());
            if(idArea!=null&&idArea>0){
            	ps.setInt(index++, idArea);	
            }
            if(fechaProximaIni!=null&&fechaProximaIni.trim().length()>0&&
            		fechaProximaFin!=null&&fechaProximaFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaProximaIni));
            	ps.setDate(index++, Util.stringToDate(fechaProximaFin));
            }
            if(fechaCargaIni!=null&&fechaCargaIni.trim().length()>0&&
            		fechaCargaFin!=null&&fechaCargaFin.trim().length()>0){
            	ps.setDate(index++, Util.stringToDate(fechaCargaIni));
            	ps.setDate(index++, Util.stringToDate(fechaCargaFin));
            }
            
            if(idEjecutivo!=null&&idEjecutivo>0){
            	ps.setInt(index++, idEjecutivo);
            }
            
            if(idEstado<9){
        		ps.setInt(index++, idEstado);
        	}
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	ReporteCorreos obj=new ReporteCorreos();
            	obj.setIdEmail(rs.getInt("id_email"));
            	obj.setSenderEmailAddress(rs.getString("SenderEmailAddress"));
            	obj.setSubjectEmail(rs.getString("SubjectEmail"));
            	obj.setdSentOn(rs.getString("dSentOn"));
            	obj.setPoseeSeguimiento(rs.getString("posee_seguimiento"));
            	obj.setCotizado(rs.getString("cotizado"));
            	obj.setFechaContacto(rs.getString("fecha_contacto"));
            	obj.setCanal(rs.getString("canal"));
            	obj.setReserva(rs.getString("reserva")); 
            	obj.setMotivo(rs.getString("motivo"));
                obj.setPaxFinal(rs.getInt("pax_final")); 
                obj.setRevenue(rs.getDouble("revenue"));
                obj.setServicioFinal(rs.getString("adset_name")); 
                obj.setEjecutivo(rs.getString("ejecutivo"));
                obj.setObservacion(rs.getString("observacion")); 
                obj.setFechaProxima(rs.getString("fecha_proxima")); 
                obj.setSeqSeguimiento(rs.getInt("seq_seguimiento"));
                obj.setIdEstadoSeguimiento(rs.getInt("id_estado_seguimiento"));
                obj.setArea(rs.getString("area"));
                
            	lista.add(obj);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	
	
	
}
