package com.delfines.spring.controller;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;

import com.delfines.dao.ConnectionFactorySQL;
import com.delfines.vo.Value;

public class LoadTripAdvisorRank {

	public static final String NBSP_IN_UTF8 = "\u00a0";
	
	public static void main(String[] args) {
	  /** CARGAR DESDE EL FORMULARIO DE CONTACTOS:INI */
	   try{
		    System.out.println("Initialize load TRIPADVISOR:: "+ZonedDateTime.now().format(DateTimeFormatter.RFC_1123_DATE_TIME));
		   
		    LoadTripAdvisorRank load=new LoadTripAdvisorRank();  
		    List<Value> lurl=load.loadHotels();
		    		
		    Iterator<Value> it = lurl.iterator();
		    
	   		while(it.hasNext()) {
	   			Value url = (Value)it.next();
				Document doc = Jsoup.connect("https://www.tripadvisor.com.pe/"+url.getDescripcion()).get();
				
				System.out.println("https://www.tripadvisor.com.pe/"+url.getDescripcion());
				boolean ready=false;
				
				for (org.jsoup.nodes.Element element : doc.getAllElements()) {
				    
					String name=element.getElementsByTag("span").attr("class");
					//if(name.trim().equals("reviewCount")){
					if(name.trim().contains("reviewCount")){
						//<span class="reviewCount">1,827 opiniones</span>
						//Element span = doc.selectFirst("span.reviewCount");
						String reviewCount=element.text().replaceAll("opiniones", "").replace(",", "").trim();
						
						//<b class="rank">N. 1</b>
						Element b = doc.selectFirst("b.rank");
						String rank=b.text().replaceAll("N.", "").trim();
						//System.out.println("   "+rank);
						
						load.updateRank(url.getId(), Integer.valueOf(rank), Integer.valueOf(reviewCount));
						
						System.out.println(" 	>> review: "+reviewCount+" rank: "+rank);
						ready=true;
					}
					
					if(name.trim().equals("hotels-hotel-review-about-with-photos-Reviews__overallRating--vElGA")){
						String rating=element.ownText();
						if(rating!=null&&rating.trim().length()>0){
							load.updateRating(url.getId(), Double.valueOf(rating.trim()));	
						}
					}
				}
				
				if(!ready){
					load.updateRank(url.getId(), Integer.valueOf(99), Integer.valueOf(0));
					System.out.println(" 	>> empty");
				}
				
	   		}
	   		System.out.println("Complete load TRIPADVISOR:: "+ZonedDateTime.now().format(DateTimeFormatter.RFC_1123_DATE_TIME));
	   }catch(Exception e){
		   e.printStackTrace();
	   }
	   /** CARGAR DESDE EL FORMULARIO DE CONTACTOS:FIN */
	}
	
	public List<Value> loadHotels(){
		List<Value> lurl=new LinkedList<Value>();
		
		Statement stmt=null;
		ResultSet rs=null;
		Connection conn=null;
		try{  
			conn = ConnectionFactorySQL.getInstance().getConnection();  
    		//here sonoo is database name, root is username and password  
    		stmt=conn.createStatement();  
    		rs=stmt.executeQuery("select id,url from dbo.hotels_tripadvisor");  
    		
    		while(rs.next()){
    			Value val=new Value();
    			val.setId(rs.getInt("id"));
    			val.setDescripcion(rs.getString("url"));
    			lurl.add(val);
    		}
    	}catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (conn != null) {try {conn.close();} catch (SQLException e) {e.printStackTrace();}}
        	if (stmt != null) {try {stmt.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
		return lurl;
	}
	
	
	public void updateRank(Integer idTripAdvisor,Integer rank,Integer reviewCount){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactorySQL.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("update dbo.general set indice=?, valor=? where id_tripadvisor=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, rank);
            ps.setInt(2, reviewCount);
            ps.setInt(3, idTripAdvisor);
            
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
	}
	
	
	public void updateRating(Integer idTripAdvisor,Double Rating){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
        try {
        	Connection conn = ConnectionFactorySQL.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("update dbo.general set puntos=? where id_tripadvisor=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setDouble(1, Rating);
            ps.setInt(2, idTripAdvisor);
            
            //--
            result=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
	}
	
}

/*
 * 
 SELECT t.indice,t.*
  FROM [db_referencias].[dbo].[general] t
  order by t.indice asc

INSERT INTO [dbo].[hotels_tripadvisor]
           ([id]
           ,[url])
     VALUES
           (13480726
           ,'Hotel_Review-g294316-d13480726-Reviews-Tierra_Viva_Miraflores_Mendiburu-Lima_Lima_Region.html')
GO

INSERT INTO [dbo].[general]
           ([denominacion]
           ,[grupo]
           ,[indice]
           ,[id_tripadvisor])
     VALUES
           ('Tierra Viva Miraflores Mendiburu'
           ,2
		   ,99
		   ,13480726
		   ) 
 * 
 */
